//+------------------------------------------------------------------+
//|                                                OrderIterator.mqh |
//|                                    Copyright 2015, EarnForex.com |
//|                                         http://www.earnforex.com |
//+------------------------------------------------------------------+
#include <OrderMap.mqh>
#include <StatusObject.mqh>
#include <RemainingOrderObject.mqh>
//+------------------------------------------------------------------+
//| Class COrderIterator                                             |
//| Purpose: Iterates through orders recursively to find minimum     |
//|          profit (maximum loss). See JS for detailed explanation. |
//+------------------------------------------------------------------+
#define UNLIMITED -DBL_MAX
#define UNDEFINED DBL_MAX

class COrderIterator
{
public:
   COrderMap *Status;
   COrderMap *RO; // Remaining orders
   double current_price;
   double unrealized_profit;
   double realized_profit;
   static double min_profit; // Same value across all instances.
   static double max_sell_volume; // Same value across all instances.
   bool skip_above;
   bool skip_below;
   double op_above;
   double op_below;

   COrderIterator(COrderMap &input_Status, COrderMap &input_RO, double input_current_price, double input_unrealized_profit, double input_realized_profit);
   COrderIterator(void): Status(NULL), RO(NULL), current_price(0), unrealized_profit(0), realized_profit(0) {Status = new COrderMap; RO = new COrderMap;}
  ~COrderIterator(void);
  
   void Iterate(double order_price);
   bool CheckSimplicity();
   void CheckPairedOrders();
   double FindPriceAbove();
   double FindPriceBelow();
   void ProcessOrder(double order_price);
   void CalculateMaxUp();
   void CalculateMaxDown();
   void AddExecutedOrderToStatus(CRemainingOrderObject &RO_order);
   void AddSLtoRO(CRemainingOrderObject &RO_order);
   void AddTPtoRO(CRemainingOrderObject &RO_order);
   void RemoveOrderFromRO(CRemainingOrderObject &RO_order);
   void RemoveOrderFromStatus(CRemainingOrderObject &RO_order);
   void CalculateUnrealizedProfit();
};

//+------------------------------------------------------------------+
//| Constructor.                                                     |
//+------------------------------------------------------------------+
COrderIterator::COrderIterator(COrderMap &input_Status, COrderMap &input_RO, double input_current_price, double input_unrealized_profit, double input_realized_profit)
{
   Status = new COrderMap; RO = new COrderMap;
   CDOMObject *order, *new_order;
   
   for (order = input_Status.GetFirstNode(); order != NULL; order = input_Status.GetNextNode())
   {
      new_order = new CStatusObject(order.Ticket(), order.Price(), order.Vol(), order.Type(), order.SL(), order.TP());
      Status.Add(new_order);
   }

   for (order = input_RO.GetFirstNode(); order != NULL; order = input_RO.GetNextNode())
   {
      new_order = new CRemainingOrderObject(order.Ticket(), order.Price(), order.Vol(), order.Type(), order.SL(), order.TP(), order.Status(), order.Origin());
      RO.Add(new_order);
   }
   
   current_price = input_current_price;
   unrealized_profit = input_unrealized_profit;
   realized_profit = input_realized_profit;
   skip_above = false;
   skip_below = false;
   op_above = UNDEFINED;
   op_below = UNDEFINED;
}


//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
COrderIterator::~COrderIterator(void)
{
   delete Status;
   delete RO;
}

// Main recursion function. Processes upper and lower orders and calls itself again.
// skip_above - if true, recursion is not used. Instead, only 'below' branch is called.
void COrderIterator::Iterate(const double order_price)
{
   #ifdef _DEBUG
      Print("Order price: ", order_price, " Current price: ", current_price);
   #endif
	if (order_price != UNDEFINED) ProcessOrder(order_price);
	CalculateUnrealizedProfit();
		
	op_above = FindPriceAbove();
	op_below = FindPriceBelow();
	
   #ifdef _DEBUG
      Print("op_above: ", op_above, " op_below: ", op_below);
   #endif

	bool skip = false;
	skip_below = false;
	// skip_above is set via class property.

	if ((op_above != UNDEFINED) && (op_below != UNDEFINED))
	{
		// Returns true if orders at the immediate price above and below the current price qualify as simple and do not require full recursion to process.
		skip = CheckSimplicity();
      #ifdef _DEBUG
		   Print("skip = ", skip);
		#endif
		// No need to do it if we are already skipping upper half via function call argument.
		if ((!skip) && (!skip_above)) 
		{
			// Check if we are inside of SL/TP of one or more orders and there is a clear minimum profit decision at this point and we won't run recursion.
			CheckPairedOrders();
			#ifdef _DEBUG
			   Print("skip_above: ", skip_above, " skip_below: ", skip_below);
			#endif
		}
	}
	
   #ifdef _DEBUG
   	Print("Current price: ", current_price);
   	Print("skip_above: ", skip_above);
   #endif   	
	if (!skip_above)
	{
		if (op_above != UNDEFINED)
		{
		   COrderIterator *OI_new_a = new COrderIterator(Status, RO, current_price, unrealized_profit, realized_profit);
		   OI_new_a.Iterate(op_above);
		   delete OI_new_a;
		}
		else
		{
			// Calculate Max Up.
			CalculateMaxUp();
		}
	}
	
	if (op_below != UNDEFINED)
	{	
		skip_above = skip;
		if (!skip_below)
		{
		   COrderIterator *OI_new_b = new COrderIterator(Status, RO, current_price, unrealized_profit, realized_profit);
		   OI_new_b.Iterate(op_below);
		   delete OI_new_b;
		}
	}
	else
	{
		// Calculate Max Down only if we do not have an indefinite loss path already. MaxDown will always be worse than indefinite loss from MaxUp.
		if (min_profit != UNLIMITED) CalculateMaxDown();
	}
}

// Checks whether orders above and below are simple and thus qualify for shorter recursion instead of full.
// Orders are considered simple if any of these conditions is true:
// 1. Plain buy/sell without SL or TP.
// 2. An SL or TP of some order, which does not have a paired TP or SL among the orders immediately below the current price.
// 3. Non-plain buy/sell in case the SL/TP does not lie within [OP1; OP2] range.
// It is enough to check SLs and TPs in the top orders. Bottom SL/TP orders will not contain any pairs for the top ones, if top ones did not have such orders.
bool COrderIterator::CheckSimplicity()
{
	CRemainingOrderObject *RO_order;
	for (RO_order = RO.GetFirstNodeAtPrice(op_above); (RO_order != NULL) && (RO_order.Price() == op_above); RO_order = RO.GetNextNode())
	{
		// If order is an SL or a TP of some other order, check if its origin has a TP or an SL in the op_below price.
		if (RO_order.Status() == SLTP)
		{
			int ticket = RO_order.Origin();
			CStatusObject *Status_order = Status.GetNodeByTicket(ticket);
			if (Status_order == NULL)
			{
			   Print(__LINE__, " Error - origin order not found by ticket: ", ticket, " Status.Total() = ", Status.Total());
			   return false;
			}
			// Partner SL/TP found - the situation is not simple.
			if ((Status_order.TP() == op_below) || (Status_order.SL() == op_below)) return(false);
		}
		else/* if (this.RO[op_above][j].status == "pending")*/
		{
			if ((RO_order.SL() != 0) && (RO_order.SL() <= op_above) && (RO_order.SL() >= op_below)) return(false);
			if ((RO_order.TP() != 0) && (RO_order.TP() <= op_above) && (RO_order.TP() >= op_below)) return(false);
		}
	}
	for (RO_order = RO.GetFirstNodeAtPrice(op_below); (RO_order != NULL) && (RO_order.Price() == op_below); RO_order = RO.GetNextNode())
	{
		if (RO_order.Status() == Pending)
		{
			if ((RO_order.SL() != 0) && (RO_order.SL() <= op_above) && (RO_order.SL() >= op_below)) return(false);
			if ((RO_order.TP() != 0) && (RO_order.TP() <= op_above) && (RO_order.TP() >= op_below)) return(false);
		}
	}
	
	return(true);
}

// Iterates through op_above orders and checks if all are SLs and TPs that have counterparts and those counterparts are located at op_below.
// Does the same for op_below orders.
// Calculates possible minimum profit for the case of going to op_above and op_below.
// Sets skips depending on where was the worst result.
void COrderIterator::CheckPairedOrders()
{
	int ticket;
	CStatusObject *Status_order;
	CRemainingOrderObject *RO_order;
	bool same_order_sl_tp = false;
	double min_profit_above = 0;
	double min_profit_below = 0;
	skip_above = false;
	skip_below = false;
	for (RO_order = RO.GetFirstNodeAtPrice(op_above); (RO_order != NULL) && (RO_order.Price() == op_above); RO_order = RO.GetNextNode())
	{
		if (RO_order.Status() == SLTP)
		{
			ticket = RO_order.Origin();
			Status_order = Status.GetNodeByTicket(ticket);
			if (Status_order == NULL)
			{
			   Print(__LINE__, " Error - origin order not found by ticket: ", ticket, " Status.Total() = ", Status.Total());
			   return;
			}			// Partner SL/TP found.
			if ((Status_order.TP() == op_below) || (Status_order.SL() == op_below))
			{
				same_order_sl_tp = true;
				// Upper order can only be an SL for a Sell.
				if (Status_order.Type() == Sell)
					min_profit_above += Status_order.Price() - op_above;
				// Upper order can only be a TP for a Buy.
				else if (Status_order.Type() == Buy)
					min_profit_above += op_above - Status_order.Price();
			}
			else return; // SL/TP without a counterpart at op_below is a different case.
		}
		else return; // Any other order would make it a much more difficult case.
	}
	// Do the same for lower orders only if upper did not fail.
	if (same_order_sl_tp) for (RO_order = RO.GetFirstNodeAtPrice(op_below); (RO_order != NULL) && (RO_order.Price() == op_below); RO_order = RO.GetNextNode())
	{
		if (RO_order.Status() == SLTP)
		{
			ticket = RO_order.Origin();
			Status_order = Status.GetNodeByTicket(ticket);
			if (Status_order == NULL)
			{
			   Print(__LINE__, " Error - origin order not found by ticket.");
			   return;
			}			// Partner SL/TP found.
			if ((Status_order.TP() == op_above) || (Status_order.SL() == op_above))
			{
				// Lower order can only be an SL for a Buy.
				if (Status_order.Type() == Buy)
					min_profit_below += op_below - Status_order.Price();
				// Lower order can only be a TP for a Sell.
				else if (Status_order.Type() == Sell)
					min_profit_below += Status_order.Price() - op_below;
			}
			else return; // SL/TP without a counterpart at op_above is a different case.
		}
		else return; // Any other order would make it a much more difficult case.
	}
	// We can now calculate the optimal direction.
	if (same_order_sl_tp)
	{
		#ifdef _DEBUG
		   Print("min_profit_above = ", min_profit_above, " min_profit_below = ", min_profit_below);
		#endif
		// Do only op_above.
		if (min_profit_above <= min_profit_below) skip_below = true;
		// Do only op_below.
		else skip_above = true;
	}
}

// Is there an order above the current price in the remaining orders.
// Returns either price or undefined.
double COrderIterator::FindPriceAbove()
{
	// Find price which is just above the current price.
	double prev_price = UNDEFINED;
	
	for (CRemainingOrderObject *RO_order = RO.GetFirstNode(); RO_order != NULL; RO_order = RO.GetNextNode())
	{
		if (RO_order.Price() < current_price) break;
		prev_price = RO_order.Price();
	}
	return(prev_price);
}

// Is there an order below the current price in the remaining orders.
double COrderIterator::FindPriceBelow()
{
	// Find op which is just below the current price.
	CRemainingOrderObject *RO_order;
	for (RO_order = RO.GetFirstNode(); RO_order != NULL; RO_order = RO.GetNextNode())
		if (RO_order.Price() < current_price) break;

	// No remaining orders below the current price.
	if ((RO_order == NULL) || (RO_order.Price() > current_price)) return(UNDEFINED);
	// Found an order price below current price.
	else return(RO_order.Price());
}

// Process one or more order, which are located at one closest price level above the current price.
void COrderIterator::ProcessOrder(double order_price)
{
	// Cycle through all remaining orders at a given price.
	for (CRemainingOrderObject *RO_order = RO.GetFirstNodeAtPrice(order_price); (RO_order != NULL) && (RO_order.Price() == order_price); RO_order = RO.GetNextNode())
	{
		// Open new order, pushing it to Status. 
		// Also removes it from RO and adds SLTP entries to RO if needed. Sorts RO if needed.
		if (RO_order.Status() == Pending)
		{
			AddExecutedOrderToStatus(RO_order);
			if (RO_order.SL()) AddSLtoRO(RO_order);
			if (RO_order.TP()) AddTPtoRO(RO_order);
		}
		// Close an executed order from the Status.
		else // Status == SLTP
		{
			// Also calculates realized profit.
			RemoveOrderFromStatus(RO_order);
		}
		RemoveOrderFromRO(RO_order);
	}
	// Switch current price.
	current_price = order_price;
}

// Calculates the minimum profit (maximum loss) attainable from going all way up with the current orders.
void COrderIterator::CalculateMaxUp()
{
	// Cycle through executed orders (Status) and calculate volume of potential sell position. If <= 0 - nothing to do.
	double sell_volume = 0;
	for (CStatusObject *Status_order = Status.GetFirstNode(); Status_order != NULL; Status_order = Status.GetNextNode())
	{
		if (Status_order.Type() == Sell) sell_volume += Status_order.Vol();
		else /*if (Status_order.Type() == Buy) */sell_volume -= Status_order.Vol();
	}
	
	if (sell_volume <= 0) return;
	
	if (sell_volume > max_sell_volume) max_sell_volume = sell_volume;
	
	unrealized_profit = UNLIMITED;
	min_profit = UNLIMITED;
}

// Calculates the minimum profit (maximum loss) attainable from going all way down with the current orders.
void COrderIterator::CalculateMaxDown()
{
	// Cycle through executed orders (Status) and calculate volume of potential buy position. If <= 0 - nothing to do.
	double buy_volume = 0;
	for (CStatusObject *Status_order = Status.GetFirstNode(); Status_order != NULL; Status_order = Status.GetNextNode())
	{
		if (Status_order.Type() == Buy) buy_volume += Status_order.Vol();
		else /*if (Status_order.Type() == Sell) */buy_volume -= Status_order.Vol();
	}
	if (buy_volume <= 0) return;
	
	// Check if we have a new max loss.
	double profit = unrealized_profit + realized_profit - buy_volume * current_price;
	if (profit < min_profit) min_profit = profit;
}

// Adds a newly executed order to the Status array.
void COrderIterator::AddExecutedOrderToStatus(CRemainingOrderObject &RO_order)
{
	CStatusObject *Status_order = new CStatusObject(RO_order.Ticket(), RO_order.Price(), RO_order.Vol(), RO_order.Type(), RO_order.SL(), RO_order.TP());
	Status.Add(Status_order);
	#ifdef _DEBUG
	   Print("Just added order #", RO_order.Ticket(), " to Status.");
	#endif
}

// Add stop-loss to the Remaining Orders array.
void COrderIterator::AddSLtoRO(CRemainingOrderObject &RO_order)
{
	CRemainingOrderObject *RO_new_order = new CRemainingOrderObject(0, RO_order.SL(), RO_order.Vol(), RO_order.Type() == Buy ? Sell : Buy, 0, 0, SLTP, RO_order.Ticket());
   RO.Add(RO_new_order);
}

// Add take-profit to the Remaining Orders array.
void COrderIterator::AddTPtoRO(CRemainingOrderObject &RO_order)
{
	CRemainingOrderObject *RO_new_order = new CRemainingOrderObject(0, RO_order.TP(), RO_order.Vol(), RO_order.Type() == Buy ? Sell : Buy, 0, 0, SLTP, RO_order.Ticket());
   RO.Add(RO_new_order);
}

// Remove an order from the Remaining Orders array.
void COrderIterator::RemoveOrderFromRO(CRemainingOrderObject &RO_order)
{
	CDOMObject * RO_order_pointer = GetPointer(RO_order);
	RO.Delete(RO.IndexOf(RO_order_pointer));
}

// Remove an RO_order's origin from the Status array and Second exit from RO (if necessary).
void COrderIterator::RemoveOrderFromStatus(CRemainingOrderObject &RO_order)
{
	CStatusObject *Status_order = Status.GetNodeByTicket(RO_order.Origin());
	#ifdef _DEBUG
	   Print(RO_order.Origin());
   	if (Status_order == NULL)
   	{
   	      Print(Status.Total());
   	   for (Status_order = Status.GetFirstNode(); Status_order != NULL; Status_order = Status.GetNextNode())
   	      Print(Status_order.Ticket());
   	}
   #endif
	// Remove second exit if any.
	if ((Status_order.SL()) && (Status_order.TP()))
	{
	   #ifdef _DEBUG
	      Print("Begin: Removing second exit of ", Status_order.Ticket(), ". SL = ", Status_order.SL(), " TP = ", Status_order.TP());
	   #endif 
	   // SL is the second exit.
	   double second_exit_price = Status_order.SL();
	   if (Status_order.TP() != RO_order.Price()) second_exit_price = Status_order.TP();
      #ifdef _DEBUG
	      Print("second_exit_price = ", second_exit_price, " Status_order.TP() = ", Status_order.TP());
	      for (CRemainingOrderObject *RO2 = RO.GetFirstNode(); RO2 != NULL; RO2 = RO.GetNextNode())
	         Print(RO2.Price());
      #endif 
      for (CRemainingOrderObject *RO_second_exit = RO.GetFirstNodeAtPrice(second_exit_price); (RO_second_exit != NULL) && (RO_second_exit.Price() == second_exit_price); RO_second_exit = RO.GetNextNode())
      {
         #ifdef _DEBUG
            Print(RO_second_exit.Origin());
         #endif 
         if (RO_second_exit.Origin() == Status_order.Ticket())
         {
            RO.DeleteCurrent();
            #ifdef _DEBUG
               Print("End: Removing second exit of ", Status_order.Ticket(), ".");
            #endif
            break;
         }
      }
   }

	if (Status_order.Type() == Buy) realized_profit += Status_order.Vol() * (RO_order.Price() - Status_order.Price());
	else/* if (Status_order.Type() == Sell) */realized_profit += Status_order.Vol() * (Status_order.Price() - RO_order.Price() - spread);
	
	#ifdef _DEBUG
	   Print("Removing ", Status_order.Ticket(), " from Status.");
	#endif 
	// Remove order from Status.
	Status.DeleteCurrent();
}

// Calculate unrealized profit based on Status array.
void COrderIterator::CalculateUnrealizedProfit()
{
	unrealized_profit = 0;
	
	for (CStatusObject *Status_order = Status.GetFirstNode(); Status_order != NULL; Status_order = Status.GetNextNode())
	{
		if (Status_order.Type() == Buy) unrealized_profit += Status_order.Vol() * (current_price - Status_order.Price());
		else /*if (Status_order.Type() == Sell)*/ unrealized_profit += Status_order.Vol() * (Status_order.Price() - current_price - spread);
		#ifdef _DEBUG
		   Print("current_price = ", current_price, " Status_order.Price() = ", Status_order.Price());
		#endif
	}
	
	#ifdef _DEBUG
	   Print("unrealized_profit = ", unrealized_profit, " realized_profit = ", realized_profit);
	#endif
	
	// Check if we have a new max loss.
	double profit = unrealized_profit + realized_profit;
	if (profit < min_profit) min_profit = profit;
}
//+------------------------------------------------------------------+