//+------------------------------------------------------------------+
//|                                               RiskCalculator.mq4 |
//|                             Copyright  2014-2016, Andriy Moraru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014-2016, Andriy Moraru, EarnForex.com"
#property link      "https://www.earnforex.com/metatrader-indicators/Risk-Calculator/"
#property version   "1.02"
#property indicator_separate_window
#property indicator_plots 0
#property strict

#property description "Calculates total risk based on the existing positions and pending orders."
#property description "Known issues:"
#property description "1. The results may be somewhat inaccurate when quote currency is different from account currency."
#property description "2. Too many pending orders with SL/TP may be too slow to process."
#property description "3. Assumes constant spread on each tick."
#property description "4. Ignores lack of margin for order execution."
#property description "5. Ignores correlations of currency pairs."
#property description "6. Ignores triangular and other forms of arbitrage."
#property description "7. Does not take into account price slippage."

#include <OrderMap.mqh>
#include <StatusObject.mqh>
#include <RemainingOrderObject.mqh>
#include <OrderIterator.mqh>

// Main
extern bool   CalculateSpreads = true; // If true, potential loss due to spreads will become the part of the potential maximum loss.
extern bool   CalculateSwaps = true; // If true, accrued swaps will become the part of the potential maximum loss.
extern double CommissionPerLot = 0; // Commission charged per lot (one side) in account currency.
extern bool   UseEquityInsteadOfBalance = false; // If true, Account Equity will be used instead of Account Balance.
// Font
extern color  cpFontColor = clrAzure; // Font color to ouput the currency pair names.
extern color  mnFontColor = clrPaleGoldenrod; // Font color to ouput the risk in money form.
extern color  pcFontColor = clrLimeGreen; // Font color to output the risk in percentage form.
extern string FontFace  = "Courier"; // Font name.
extern int    FontSize  = 8; // Font size.
// Spacing
extern int    scaleY = 15; // Number of pixels per line in output.
extern int    offsetX = 20; // Horizontal offset for output.
extern int    offsetY = 20; // Vertical offset for output.

// Main object for calculating minimum profit (maximum loss) with its static variables initialized.
double COrderIterator::min_profit = UNDEFINED;
double COrderIterator::max_sell_volume = 0;
COrderIterator *OrderIterator;

// Global variable for the current output line's vertical indent.
int Y;
// Global variable for the separate indicator window number;
int Window = -1;

// Global variable to store loss due to swaps of the currently open positions. In account currency.
double swap;
// Global variable for spread - it is used in OrderIterator instances but is defined for each currency pair in this file.
double spread;
// Global variable to store loss due to commission that will be incurred based on current trades. In account currency.
double commission;
// Global variable to store the detected account currency.
string AccCurrency;

#ifdef _DEBUG   
   bool single_run = false;
#endif

//+------------------------------------------------------------------+
//| Initialization function.                                         |
//+------------------------------------------------------------------+
int OnInit()
{
   IndicatorShortName("Risk Calculator");
   return(0);
}

//+------------------------------------------------------------------+
//| Deinitialization function.                                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   ObjectsDeleteAll(Window, OBJ_LABEL);
}

//+------------------------------------------------------------------+
//| Script program start function.                                   |
//+------------------------------------------------------------------+
int start()
{
   // If could not find account currency, probably not connected.
   AccCurrency = AccountCurrency();
   if (AccCurrency == "") return(0);

   CalculateRisk();
   
   return(0);
}

//+------------------------------------------------------------------+
//| Trade event handler.                                             |
//+------------------------------------------------------------------+
void OnTrade()
{
   CalculateRisk();
}

//+------------------------------------------------------------------+
//| Main calculation function. Called from event handlers.           |
//+------------------------------------------------------------------+
void CalculateRisk()
{
   #ifdef _DEBUG   
      if (single_run) return;
   #endif 
   
   double total_risk = 0;
   if (Window == -1) Window = WindowFind("Risk Calculator");

   Y = 0;

   int total = OrdersTotal();
   ObjectsDeleteAll(Window, OBJ_LABEL);
   for (int i = 0; i < total; i++)
   {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;

      string cp = OrderSymbol();

      // This currency pair has already been processed - there is a TEXT_LABEL with its name.
      if (ObjectFind(0, cp) >= 0) continue;

      // Zero swap loss before proceeding to the next currency pair.
      swap = 0;
      commission = 0;
      
      double risk = ProcessCurrencyPair(cp);
      // UNDEFINED is stronger than UNLIMITED. UNLIMITED is stronger than any number.
      if ((risk != UNLIMITED) && (total_risk != UNLIMITED) && (risk != UNDEFINED) && (total_risk != UNDEFINED)) total_risk += risk;
      else if (risk == UNDEFINED) total_risk = UNDEFINED;
      else if (total_risk != UNDEFINED) total_risk = UNLIMITED;
   }
   
   OutputTotalRisk(total_risk);
   
   #ifdef _DEBUG   
      single_run = true;
   #endif
}

//+------------------------------------------------------------------+
//| Calculates and outputs risk for a given currency pair.           |
//+------------------------------------------------------------------+
double ProcessCurrencyPair(string cp)
{
   COrderIterator::min_profit = UNDEFINED;
   COrderIterator::max_sell_volume = 0;
	OrderIterator = new COrderIterator();

   CDOMObject *order;
   
   // Maximum loss (risk) in account currency.
   double MoneyRisk = 0;
   
   if (CalculateSpreads) spread = SymbolInfoInteger(cp, SYMBOL_SPREAD) * SymbolInfoDouble(cp, SYMBOL_POINT);
   else spread = 0;
   
   int total = OrdersTotal();
   
   for (int i = 0; i < total; i++)
   {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;

      if (OrderSymbol() != cp) continue;
      
      double volume = OrderLots();
      double sl = OrderStopLoss();
      double tp = OrderTakeProfit();
      
      if (OrderType() == ORDER_TYPE_BUY)
      {
         if (CalculateSwaps) swap += OrderSwap();
         commission += CommissionPerLot * volume;
         order = new CStatusObject(OrderTicket(), OrderOpenPrice(), volume, Buy, sl, tp);
         OrderIterator.Status.Add(order);
         if (sl)
         {
            order = new CRemainingOrderObject(0, sl, volume, Sell, 0, 0, SLTP, OrderTicket());
            OrderIterator.RO.Add(order);
         }
         if (tp)
         {
            order = new CRemainingOrderObject(0, tp, volume, Sell, 0, 0, SLTP, OrderTicket());
            OrderIterator.RO.Add(order);
         }
      }
      else if (OrderType() == ORDER_TYPE_SELL)
      {
         if (CalculateSwaps) swap += OrderSwap();
         commission += CommissionPerLot * volume;
         order = new CStatusObject(OrderTicket(), OrderOpenPrice(), volume, Sell, sl == 0 ? 0 : sl - spread, tp == 0 ? 0 : tp - spread);
         OrderIterator.Status.Add(order);
         if (sl)
         {
            order = new CRemainingOrderObject(0, sl == 0 ? 0 : sl - spread, volume, Buy, 0, 0, SLTP, OrderTicket());
            OrderIterator.RO.Add(order);
         }
         if (tp)
         {
            order = new CRemainingOrderObject(0, tp == 0 ? 0 : tp - spread, volume, Buy, 0, 0, SLTP, OrderTicket());
            OrderIterator.RO.Add(order);
         }
      }
      else
      {
         commission += CommissionPerLot * volume * 2;
         if ((OrderType() == ORDER_TYPE_SELL_LIMIT) || (OrderType() == ORDER_TYPE_SELL_STOP))
         {
            order = new CRemainingOrderObject(OrderTicket(), OrderOpenPrice(), volume, Sell, sl == 0 ? 0 : sl - spread, tp == 0 ? 0 : tp - spread, Pending, 0);
            OrderIterator.RO.Add(order);
         }
         else if ((OrderType() == ORDER_TYPE_BUY_STOP) || (OrderType() == ORDER_TYPE_BUY_LIMIT))
         {
            order = new CRemainingOrderObject(OrderTicket(), OrderOpenPrice() - spread, volume, Buy, sl, tp, Pending, 0);
            OrderIterator.RO.Add(order);
         }
      }
   }      
   
   OrderIterator.current_price = SymbolInfoDouble(cp, SYMBOL_BID);

	OrderIterator.Iterate(UNDEFINED);

   delete OrderIterator;

   MoneyRisk = Output(cp);

   return(MoneyRisk);
}

//+------------------------------------------------------------------+
//| Creates output for one currency pair via a graphical object.     |
//| Returns: risk in account currency.                               |
//+------------------------------------------------------------------+
double Output(string cp)
{
   string RiskOutput = "", SecondRiskOutput = "";
   double MoneyRisk = 0;
   if (COrderIterator::min_profit == UNDEFINED) RiskOutput = JustifyRight("Undefined", 25);
   else if (COrderIterator::min_profit == UNLIMITED) RiskOutput = JustifyRight("Unlimited", 25) +  " (" + DoubleToString(COrderIterator::max_sell_volume, 2) + " lot)";
   else
   {
      double UnitCost = MarketInfo(cp, MODE_TICKVALUE);
      double OnePoint = MarketInfo(cp, MODE_POINT);
      MoneyRisk = -COrderIterator::min_profit * UnitCost / OnePoint - commission;
      if (CalculateSwaps) MoneyRisk -= swap;
      double Size;
      if (UseEquityInsteadOfBalance) Size = AccountEquity();
      else Size = AccountBalance();
      double PercentageRisk = (MoneyRisk / Size) * 100;
      RiskOutput = JustifyRight(FormatNumber(DoubleToString(MoneyRisk, 2)) + " " + AccCurrency, 25);
      SecondRiskOutput = JustifyRight(DoubleToString(PercentageRisk, 2) + "%", 25);
   }
      
   ObjectCreate(cp, OBJ_LABEL, Window, 0, 0);
   ObjectSetText(cp, cp, FontSize, FontFace, cpFontColor);
   ObjectSet(cp, OBJPROP_CORNER, 0);
   ObjectSet(cp, OBJPROP_XDISTANCE, offsetX);
   ObjectSet(cp, OBJPROP_YDISTANCE, Y * scaleY + offsetY);
   
   ObjectCreate(cp + "Amount", OBJ_LABEL, Window, 0, 0);
   ObjectSetText(cp + "Amount", RiskOutput, FontSize, FontFace, mnFontColor);
   ObjectSet(cp + "Amount", OBJPROP_CORNER, 0);
   ObjectSet(cp + "Amount", OBJPROP_XDISTANCE, offsetX + 1 * offsetX);
   ObjectSet(cp + "Amount", OBJPROP_YDISTANCE, Y * scaleY + offsetY);

   if (SecondRiskOutput != "")
   {
      ObjectCreate(cp + "Percentage", OBJ_LABEL, Window, 0, 0);
      ObjectSetText(cp + "Percentage", SecondRiskOutput, FontSize, FontFace, pcFontColor);
      ObjectSet(cp + "Percentage", OBJPROP_CORNER, 0);
      ObjectSet(cp + "Percentage", OBJPROP_XDISTANCE, offsetX + 6 * offsetX);
      ObjectSet(cp + "Percentage", OBJPROP_YDISTANCE, Y * scaleY + offsetY);
   }
   
   Y++;
   
   if ((COrderIterator::min_profit != UNLIMITED) && (COrderIterator::min_profit != UNDEFINED)) return(MoneyRisk);
   else return(COrderIterator::min_profit);
}

//+------------------------------------------------------------------+
//| Creates output for combined risk for all currency pairs.         |
//| Inputs: risk - risk in account currency.                         |
//+------------------------------------------------------------------+
void OutputTotalRisk(double risk)
{
   Y++;
   string RiskOutput = "", SecondRiskOutput = "";;
   if (risk == 0) RiskOutput = JustifyRight("No risk.", 25);
   else if (risk == UNDEFINED) RiskOutput = JustifyRight("Undefined", 25);
   else if (risk == UNLIMITED) RiskOutput = JustifyRight("Unlimited", 25);
   else
   {
      double Size;
      if (UseEquityInsteadOfBalance) Size = AccountEquity();
      else Size = AccountBalance();
      double PercentageRisk = (risk / Size) * 100;
      //RiskOutput = DoubleToString(risk, 2) + " (" + DoubleToString(PercentageRisk, 2) + "%)";
      RiskOutput = JustifyRight(FormatNumber(DoubleToString(risk, 2)) + " " + AccCurrency, 25);
      SecondRiskOutput = JustifyRight(DoubleToString(PercentageRisk, 2) + "%", 25);
   }
      
   ObjectCreate("Total", OBJ_LABEL, Window, 0, 0);
   ObjectSetText("Total", "Total", FontSize, FontFace, cpFontColor);
   ObjectSet("Total", OBJPROP_CORNER, 0);
   ObjectSet("Total", OBJPROP_XDISTANCE, offsetX);
   ObjectSet("Total", OBJPROP_YDISTANCE, Y * scaleY + offsetY);

   ObjectCreate("TotalAmount", OBJ_LABEL, Window, 0, 0);
   ObjectSetText("TotalAmount", RiskOutput, FontSize, FontFace, mnFontColor);
   ObjectSet("TotalAmount", OBJPROP_CORNER, 0);
   ObjectSet("TotalAmount", OBJPROP_XDISTANCE, offsetX + 1 * offsetX);
   ObjectSet("TotalAmount", OBJPROP_YDISTANCE, Y * scaleY + offsetY);

   if ((SecondRiskOutput != "") && (risk != 0))
   {
      ObjectCreate("TotalPercentage", OBJ_LABEL, Window, 0, 0);
      ObjectSetText("TotalPercentage", SecondRiskOutput, FontSize, FontFace, pcFontColor);
      ObjectSet("TotalPercentage", OBJPROP_CORNER, 0);
      ObjectSet("TotalPercentage", OBJPROP_XDISTANCE, offsetX + 6 * offsetX);
      ObjectSet("TotalPercentage", OBJPROP_YDISTANCE, Y * scaleY + offsetY);
   }
}

//+----------------------------------------------------------------------+
//| Returns a number formatted as money string.                          |
//+----------------------------------------------------------------------+
string FormatNumber(const string number)
{
   string output = "";
   int length = StringLen(number);
   int j = 0;
   // Start from first digit from right starting from the decimal separator. E.g. from '4' in '1234.56'.
   for (int i = length - 4; i >= 0; i--)
   {
      if ((j % 3 == 0) && (j != 0)) output = "," + output;
      output = StringSubstr(number, i, 1) + output;
      j++;
   }
   return(output + StringSubstr(number, length - 3, 3));
}

//+----------------------------------------------------------------------+
//| Returns a string with enough spaces added from the left side to make | 
//| it a width length string.                                            |
//+----------------------------------------------------------------------+
string JustifyRight(string text, const int width)
{
   int length = StringLen(text);
   // Cannot do anything - string too long.
   if (length >= width) return(text);
   
   int j = 0;
   // Start from first digit from right starting from the decimal separator. E.g. from '4' in '1234.56'.
   for (int i = width - length; i >= 0; i--)
      text = " " + text;
   
   return(text);
}